@@ -1,12 +1,12 @@
-__version__ = "1.0.20"
+__version__ = "1.0.24"
 
+from .functional.convert import from_primitive, to_primitive
+from .functional.dotlist import from_dotlist, to_dotlist
+from .functional.load import from_string, from_file_uri, from_cli
+from .functional.merge import merge
+from .functional.util import mask_in, mask_out, apply
+from .resolvers.collection_resolvers.collection_resolver import CollectionResolver
 from .resolvers.default_resolver import DefaultResolver
+from .resolvers.processors.processor import Processor
 from .resolvers.resolver import Resolver
 from .resolvers.scalar_resolvers.scalar_resolver import ScalarResolver
-from .resolvers.collection_resolvers.collection_resolver import CollectionResolver
-from .resolvers.processors.processor import Processor
-
-from .functional.load import from_string, from_file_uri, from_cli
-from .functional.util import merge, mask_in, mask_out, apply
-from .functional.dotlist import from_dotlist, to_dotlist
-from .functional.convert import from_primitive, to_primitive
\ No newline at end of file
