@@ -1,8 +1,9 @@
+from copy import deepcopy
+
+from ..entities.grammar_tree_nodes import RootNode, FixedNode, InterpolatedNode
 from ..entities.wrappers import KCDict, KCList, KCScalar, KCObject
 from ..grammar.scalar_grammar import parse_scalar
-from ..entities.grammar_tree_nodes import RootNode, FixedNode, InterpolatedNode
-from ..functional.util import trace_to_full_accessor
-from copy import deepcopy
+
 
 class Resolver:
     def __init__(
@@ -17,7 +18,6 @@ class Resolver:
         self.pre_processors = pre_processors or []
         self.post_processors = post_processors or []
 
-
     def resolve(self, node, root_node=None):
         node = deepcopy(node)
         if root_node is None:
@@ -27,7 +27,8 @@ class Resolver:
         result = {}
         root_node_to_pass = pre_processed if root_node is None else root_node
         wrapped_node = KCDict(root=pre_processed)
-        self._resolve_collection(pre_processed, root_node=root_node_to_pass, result=result, trace=[(wrapped_node, "root")])
+        self._resolve_collection(pre_processed, root_node=root_node_to_pass, result=result,
+                                 trace=[(wrapped_node, "root")])
         processed_result = result["root"]
         # only postprocess from root call (e.g. template resolver also calls resolve but with a root_node parameter)
         if root_node is None:
@@ -36,12 +37,15 @@ class Resolver:
             post_processed = processed_result
         return post_processed
 
-
     def _resolve_collection(self, node, root_node, result, trace):
         parent, parent_accessor = trace[-1]
         if isinstance(node, KCDict):
             if isinstance(parent_accessor, int):
-                result.append({})
+                if isinstance(result, list):
+                    result.append({})
+                else:
+                    # dict with int index
+                    result[parent_accessor] = {}
             else:
                 result[parent_accessor] = {}
             # preorder
@@ -61,7 +65,7 @@ class Resolver:
                 resolver.postorder_resolve(node, root_node=root_node, result=result, trace=trace, root_resolver=self)
         elif isinstance(node, KCList):
             if isinstance(parent_accessor, int):
-                result[parent_accessor].append([])
+                result.append([])
             else:
                 result[parent_accessor] = []
             # preorder
@@ -90,10 +94,17 @@ class Resolver:
                 resolve_result = node.value
             else:
                 resolve_result = node.value
+                # track resolved scalars to avoid endless loops in resolving (e.g. prop: ${prop})
+                scalar_resolver_trace = []
                 while True:
+                    scalar_resolver_trace.append(resolve_result)
                     resolve_result = self.resolve_scalar(resolve_result, root_node=root_node, trace=trace)
                     if not isinstance(resolve_result, str) or not self._requires_resolve_scalar(resolve_result):
                         break
+                    # check for recursive resolving
+                    if resolve_result in scalar_resolver_trace:
+                        from ..errors import recursive_resolving_error
+                        raise recursive_resolving_error(trace)
 
             # resolved value might be a KCObject (e.g. when loading a nested yaml)
             if isinstance(resolve_result, KCObject):
@@ -101,8 +112,7 @@ class Resolver:
                 self._resolve_collection(resolve_result, root_node=root_node, result=result, trace=trace)
             else:
                 # set value
-                parent_accessor = trace[-1][1]
-                if isinstance(parent_accessor, int):
+                if isinstance(parent_accessor, int) and isinstance(parent, KCList):
                     if len(result) != parent_accessor:
                         raise IndexError
                     result.append(resolve_result)
@@ -128,19 +138,27 @@ class Resolver:
 
     def _resolve_scalar(self, grammar_node, root_node, trace):
         if isinstance(grammar_node, RootNode):
-            resolve_results = [self._resolve_scalar(child, root_node=root_node, trace=trace) for child in grammar_node.children]
+            resolve_results = [
+                self._resolve_scalar(child, root_node=root_node, trace=trace)
+                for child in grammar_node.children
+            ]
             return self._merge_scalar_resolve_results(resolve_results)
         elif isinstance(grammar_node, FixedNode):
             return grammar_node.value
         elif isinstance(grammar_node, InterpolatedNode):
             # resolve children
-            resolve_results = [self._resolve_scalar(child, root_node=root_node, trace=trace) for child in grammar_node.children]
+            resolve_results = [
+                self._resolve_scalar(child, root_node=root_node, trace=trace)
+                for child in grammar_node.children
+            ]
             resolve_result = self._merge_scalar_resolve_results(resolve_results)
             # resolve cur node
             if grammar_node.resolver_key not in self.scalar_resolvers:
                 from ..errors import invalid_resolver_key
-                raise invalid_resolver_key(grammar_node.resolver_key, list(self.scalar_resolvers.keys()),
-                                           trace_to_full_accessor(trace), root_node.source_id)
+                raise invalid_resolver_key(
+                    grammar_node.resolver_key, list(self.scalar_resolvers.keys()),
+                    trace, root_node.source_id
+                )
             scalar_resolver = self.scalar_resolvers[grammar_node.resolver_key]
             resolved_scalar = scalar_resolver.resolve(resolve_result, root_node=root_node, trace=trace)
             return resolved_scalar
