@@ -1,8 +1,11 @@
-import unittest
 import os
-import kappaconfig as kc
+import unittest
+
 import yaml
 
+import kappaconfig as kc
+
+
 class TestComplexYamls(unittest.TestCase):
     @staticmethod
     def get_file_uris(root_path):
@@ -18,7 +21,6 @@ class TestComplexYamls(unittest.TestCase):
                 file_uris += [file_or_dir_uri]
         return file_uris
 
-
     def test_testcase_exists_for_every_yaml_file(self):
         # there probably are libraries that allow this implicitly (e.g. ddt should be able to do this)
         # but this makes running/debugging individual tests a hassle, so just check it manually
@@ -45,6 +47,9 @@ class TestComplexYamls(unittest.TestCase):
     def test_data_mvtec(self):
         self.resolve_yaml("data/mvtec.yaml")
 
+    def test_data_toy(self):
+        self.resolve_yaml("data/toy.yaml")
+
     def test_loggers_default_epochs(self):
         self.resolve_yaml("loggers/default_epochs.yaml")
 
